import { nanoid } from "nanoid";
import { createDiamond, createOss, createTexture, createUV } from "./create";
import { RxValue } from "@/comm/controllers/rxValue";

export class PackMat {
  id = nanoid();
  generateType?: "colormatch" | "tech" | undefined;
  userData: any;

  state = RxValue.create({
    name: "未命名",
    cusNum: "编号",
    thumbnail: createOss(),
    classType: "pbr" as "pbr" | "diamond" | "cuscolor",
    type: "meta" as "meta" | "spec",
    uvMap: "box" as "box" | "uv",
    cullFace: "",
    diamond: createDiamond(),
    normal: createTexture({
      factor: 1,
    }),
    metalness: createTexture({
      factor: 0,
    }),
    roughness: createTexture({
      factor: 1,
    }),
    gloss: createTexture({
      factor: 1,
    }),
    albedo: createTexture({
      color: [0, 1, 0],
    }),
    diffuse: createTexture({
      color: [0, 1, 0],
    }),

    specular: createTexture({ 
      color: [1, 1, 1],
    }),
    displace: createTexture({
      factor: 1,
      enable: false,
    }),
    opacity: createTexture({
      factor: 1,
      enable: false,
    }),
    uv: createUV(),
    cusUv: createUV(),
  })
  // colorMatch?: any;

  get uv() {
    return this.state.uv;
  }
  get cusUv() {
    return this.state.cusUv;
  }
  
  get albedo() {
    return this.state.albedo;
  }
  get diffuse() {
    return this.state.diffuse;
  }
  
  get normal() {
    return this.state.normal;
  }
  get metalness() {
    return this.state.metalness;
  }
  get name() {
    return this.state.name;
  }
  
  get roughness() {
    return this.state.roughness;
  }
  
  toJson() {
    let out = {
      id: this.id, generateType: this.generateType, userData: this.userData,
      ...this.state.toJson()
    }
    return out;
  }
  fromJson( data:any ) {
    this.id = data.id;
    this.generateType = data.generateType;
    this.userData = data.userData;
    this.state.fromJson(data);
  }
}