import { nanoid } from "nanoid";
import { createOss } from "./create";
import { RxValue } from "@/comm/controllers/rxValue";

export class CraftImage {
  id =  nanoid();
  state = RxValue.create({
    width: 0,
    height: 0,
    image: { url: "", size: 0 },
    position: [0, 0],
    scale: [1, 1],
    rotation: 0,
    fillType: "tz" as "tz" | "mb", //贴图方式 tz 贴纸 mb 满版
    technology: "ss" as "ss" | "tj" | "ty" | "at" | "ls" | "uv" //工艺类型 ss四色 tj烫金 ty烫银 at凹凸 ls镭射 uvUV
  })
  get width() {
    return this.state.width;
  }
  get height() {
    return this.state.height;
  }
  
  get image() {
    return this.state.image;
  }
  get technology () {
    return this.state.technology;
  }
  get rotation () {
    return this.state.rotation;
  }
  get fillType () {
    return this.state.fillType;
  }
  get scale () {
    return this.state.scale;
  }
  get position () {
    return this.state.position;
  }
  
  toJson() {
    return {
      id: this.id,
      ...this.state.toJson()
    }
  }
  fromJson( data:any) {
    this.id = data.id;
    this.state.fromJson( data );
  }
}

export class PackProductComp {
  id = "mustfromMeshCompId"
  userData: any;

  state = RxValue.create({
    name : "",
    matId: "",
    groupId: "",  //组件打组
    uvMap: createOss(),
    uvsize: { width: 0, height: 0 },
    visible: true,
    locked: false, //当前部件是否被锁定
    images:  [] as CraftImage[] //工艺贴图
  })
  
  get images () {
    return this.state.images || [];
  }
  get matId() {
     return this.state.matId;
  }
  
  toJson() {
    const out :any = {
      id: this.id,
      userData: this.userData,
      ...this.state.toJson()
    }
    out.images = this.state.images.map(item=>item.toJson())
    return out;
  }
  
  fromJson(data:any) {
    this.id = data.id;
    this.userData = data.userData;
    if (!data.images) data.images = [];
    this.state.fromJson(data);
    this.state.images = data.images.map((item:any)=>{
       const s = new CraftImage();
       s.fromJson(item);
       return s;
    })
  }

  addImage(uri:string, width: number, height: number, Size:number) {
      let m = new CraftImage();
      m.state.image.url = uri;
      m.state.image.size = Size;
      m.state.width = width;
      m.state.height = height;
      let images = this.state.images.slice(0)
      images.push(m);
      this.state.setImages(images);
  }

  removeImage(id:string) {
    let n = this.state.images.length;
    let images = this.state.images.slice(0)
    while(n--) {
      if(images[n].id == id) {
        images.splice(n, 1);
        break
      }
    }
    this.state.setImages(images);
  }
}
