import { nanoid } from "nanoid";
import { createBackground, createOss } from "./create";
import { PackSceneLight } from "./PackSceneLight";
import { PackSceneProduct } from "./PackSceneProduct";
import { RxValue } from "@/comm/controllers/rxValue";

export class PackScene {
  id = nanoid();
  name = "";
  thumbnail = createOss();
  userData: any;
  stickers: any[] = [];

  state = RxValue.create({
    envId: "",
    lights: {
      enable: false,
      list: [] as PackSceneLight[],
    },
    products: [] as PackSceneProduct[],
    background: createBackground(),
    options: {
      rotation: 0,
      exposure: 1,
    },
    toneMap: {
      method: 1, //0 "linear" 值0 "reinhard" 值 1 "filmic”:值2
      exposure: 1.0,
      brightness: 0.1,
      contrast: 0.06,
      saturation: 1,
    }
  })
  get products() {
    return this.state.products;
  }
  get envId() {
    return this.state.envId;
  }

  

  toJson() {
    return {
      id: this.id,
      name: this.name,
      thumbnail: this.thumbnail,
      userData: this.userData,
      stickers: this.stickers,
      ...this.state.toJson(),
      products: this.state.products.map(item=>item.toJson()),
    }
  }

  fromJson(data:any) {
    this.id = data.id;
    this.name = data.name;
    this.thumbnail = data.thumbnail;
    this.userData = data.userData;
    this.stickers = data.stickers;
    this.state.fromJson(data);
    this.state.products = (data.products || []).map((item:any)=>{
       const s = new PackSceneProduct();
       s.fromJson(item);
       return s;
    })
  }
}
