import { Dict_Imgs } from "@/dict";
import { createAttrsForm } from "../../defines/createAttrsForm";
import { InputNumber, Switch } from "ant-design-vue";
import { createColorOpts } from "../../defines/formOpts/createColorOpts";
import { eagerComputed } from "@vueuse/core";
import { DesignCompObj } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { RxValue } from "@/modules/editor/controllers/ReactCtrl/rxValue";

export { Component } from "./component";

export const options = {
  name: "连续曲线",
  thumbnail: require("@/modules/editor/assets/icons/curve.svg"),
};

const CurveValue = {
  lineColor: "#000000",
  lineWidth: 1,
  isFill: false,
  fillColor: "#000000",
  reverseFill: false,
  points: [] as any[],
  isClose: true,
  isCurve: false,
}

export class CompCurveObj extends DesignCompObj<typeof CurveValue> {
  value = RxValue.create({...CurveValue})
  onCreated() {
    this.compKey = "Curve";
    this.layout.size = [750, 750];
  }
  fromJson(data:any) {
    super.fromJson(data);

    console.log("--------------", this.value.points);
    let hasEmpty = false;
    this.value.points.forEach(item=>{
      if (!item) hasEmpty = true;
    })
    if (hasEmpty) this.value.points = [];
  }
}

export function createComp() {
  return new CompCurveObj();
}


export const Form = createAttrsForm([
  {
    label: "线宽",
    dataIndex: "value.lineWidth",
    component: InputNumber,
  },
  {
    label:"颜色",
    dataIndex: "value.lineColor",
    ...createColorOpts(),
  },
  {
    label: "是否填充",
    dataIndex: "value.isFill",
    component: "Switch",
  },
  {
    label:"填充颜色",
    dataIndex: "value.fillColor",
    ...createColorOpts(),
    isVisible: (value, data) => data?.value?.isFill == true,
  },
  {
    label:"反向填充",
    dataIndex: "value.reverseFill",
    component: "Switch",
    isVisible: (value, data) => data?.value?.isFill == true,
  },
  {
    label: "是否封闭",
    dataIndex: "value.isClose",
    component: "Switch",
  },
]);
