import { createAttrsForm } from "../../defines/createAttrsForm";
import { InputNumber, Switch } from "ant-design-vue";
import { createColorOpts } from "../../defines/formOpts/createColorOpts";
import Slider from "../../formItems/Slider";
import { DesignCompObj } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { RxValue } from "@/modules/editor/controllers/ReactCtrl/rxValue";

export { Component } from "./component";

export const options = {
  name: "椭圆",
  thumbnail: require("@/modules/editor/assets/icons/ellicipe.svg"),
};

const EllipseValue = {
  lineColor: "#000000",
  lineWidth: 1,
  isFill: false,
  fillColor: "#000000",
  reverseFill: false,
  start: 0,
  end: 360
}
export class CompEllipseObj extends DesignCompObj<typeof EllipseValue> {
  value = RxValue.create({...EllipseValue})
  
  onCreated() {
    this.compKey = "Ellipse";
    this.layout.size = [200, 400];
  }
}
export function createComp() {
  return new CompEllipseObj();
}

export const Form = createAttrsForm([
    {
        label: "线宽",
        dataIndex: "value.lineWidth",
        component: InputNumber,
    },
      {
        label:"颜色",
        dataIndex: "value.lineColor",
        ...createColorOpts(),
      },
      {
        label: "是否填充",
        dataIndex: "value.isFill",
        component: "Switch",
      },
      {
        label:"填充颜色",
        dataIndex: "value.fillColor",
        ...createColorOpts(),
        isVisible: (value, data) => data?.value?.isFill == true,
      },
      {
        label:"反向填充",
        dataIndex: "value.reverseFill",
        component: "Switch",
        isVisible: (value, data) => data?.value?.isFill == true,
      },
      {
        label: "起角",
        dataIndex: "value.start",
        component: Slider,
        props: {
            defaultValue: 0,
            min: 0,
            max: 360,
            step: 1,
          },
    },
    {
        label: "终角",
        dataIndex: "value.end",
        component: Slider,
        props: {
            defaultValue: 180,
            min: 0,
            max: 360,
            step: 1,
          },
    },
]);
