import { css, cx } from "@linaria/core";
import "swiper/css";
// import "swiper/css/navigation";
import "swiper/css/pagination";
import { defineComponent, nextTick, onMounted, onUnmounted } from "vue";
import { string } from "vue-types";

import { IconArrowUp, IconNext, IconPrev } from "@/assets/icons";
import { useResource } from "@/modules/resource";
import { CompPageObj } from ".";
import { useEditor } from "../../../..";
import { useCompData } from "../../defines/hook";
import { useCompRef } from "../hooks";
import { PageMusic } from "./PageMusic";

export const Component = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props, { slots }) {
    const editor = useEditor();
    const { helper, store, controls } = editor;
    const compRef = useCompRef(props.compId);
    const resource = useResource();
    const ctrl = resource.controls.materialAudioListCtrl;
    const getCustomMusic = async () => {
      await ctrl.loadPage(1, 1);
      if (ctrl.state.list.length > 0) {
        const item: any = ctrl.state.list[0];
        const fileUrl = item.file.url;
        const option = {
          label: "自定义",
          value: "custom",
          src: fileUrl,
        };
        const options = [...controls.mediaCtrl.state.musicOptions, option];
        controls.mediaCtrl.setMusicOptions(options);
      }
    };
    getCustomMusic();
    // const getContextStyles = () => {
    //   let contextStyle: any = null;
    //   if (store.rootPage.value.pageMode == "short" && !store.isEditMode) {
    //     contextStyle = {
    //       transform: `translate(0, ${
    //         -store.shortPage.index * window.innerHeight + store.shortPage.offset
    //       }px`,
    //     };
    //     if (!store.shortPage.isMoving) {
    //       contextStyle.transition = "transform .4s ease-out";
    //     }
    //   }
    //   return contextStyle;
    // };

    const getPageStyles = () => {
      const comp = useCompData<CompPageObj>(props.compId);
      const { layout, value } = comp;

      const style = helper.createStyle(layout || { size: [750] }, comp);
      if (!store.isDisplay) {
        style.overflow = "unset";
        return style;
      }

      const isPcDesign = value.useFor == "pc";
      const isShortDesign = comp.value.pageMode == "short";

      // 翻页
      if (isShortDesign) {
        if (!isPcDesign) {
          style.height = "100%";
        } else {
          style.height = "100vh";
        }
        if (isPcDesign) {
          style.width = "100vw";
        }
      } else {
        // 长页
        style.height = "auto";
        if (isPcDesign) {
          // pc
          style.margin = "0 auto";
        }
      }
      // ?
      // style.transform = "matrix(1,0,0,1,0,0)";
      return style;
    };

    const isShowSwiper = () => {
      const { value } = useCompData<CompPageObj>(props.compId);
      return (
        value.pageMode == "short" &&
        // value.useFor == "pc" &&
        !editor.store.isEditMode
      );
    };

    onMounted(() => {
      if (isShowSwiper()) controls.previewCtrl.initSwiper(compRef.value);
    });

    onUnmounted(() => {
      controls.previewCtrl.destroyEvent();
    });

    return () => {
      const comp = useCompData<CompPageObj>(props.compId);
      const { children, value } = comp;
      const isPcDesign = value.useFor == "pc";

      const compMusic = value?.music || "";
      const curValue = controls.mediaCtrl.state.musicOptions.find((e) => {
        return e.value == compMusic;
      });

      const showSwiper = isShowSwiper();
      // const contextStyle = getContextStyles();
      const pageStyles = getPageStyles();
      return (
        <div
          key={controls.pageCtrl.state.designId}
          ref={compRef}
          style={pageStyles}
          class={cx(
            "page",
            "relative",
            pageRootStyles,
            editor.store.isEditMode && pageEditStyle
          )}
        >
          {(store.isDisplay ||
            (store.isEditMode &&
              controls.screenCtrl.state.screen.pageMode != "short")) && (
            <div
              class={cx("relative z-10", showSwiper && "swiper-wrapper")}
              // style={contextStyle}
            >
              {slots.Container?.(
                children.default.map((compId) => {
                  const comp = helper.findComp(compId);
                  if (!comp) return;
                  return slots.CompItem?.(comp);
                })
              )}
            </div>
          )}

          {store.isEditMode &&
            controls.screenCtrl.state.screen.pageMode == "short" &&
            controls.pageCtrl.currStreamCard && (
              // <div
              //   class="relative z-1000"
              //   // style={contextStyle}
              // >
              // </div>
              <>{slots.Container?.([slots.CompItem?.(controls.pageCtrl.currStreamCard)])}</>
            )}

          {/* 轮播组件 */}
          {showSwiper && (
            <>
              <div class="swiper-pagination"></div>
              {!isPcDesign && (
                <div class="mobile_director flex items-center justify-center">
                  <IconArrowUp
                    class={cx(iconStyles, "!text-light-50 text-16px")}
                  />
                </div>
              )}
              {isPcDesign && (
                <div class="absolute bottom-30px left-1/2 w-130px h-44px px-4px flex items-center justify-between bg-[#262626] rounded-4px transform -translate-x-1/2 z-1">
                  <IconPrev class="swiper-button-prev rounded-4px text-36px cursor-pointer transition hover:bg-dark-600 active:bg-dark-900" />
                  <span class="select-none">
                    {controls.previewCtrl.state.activeIndex}/
                    {children.default.length}
                  </span>
                  <IconNext class="swiper-button-next rounded-4px text-36px cursor-pointer transition hover:bg-dark-600 active:bg-dark-900" />
                </div>
              )}
            </>
          )}
          {/* music */}
          {curValue?.value && !editor.store.isEditMode && <PageMusic />}
        </div>
      );
    };
  },
});

const iconStyles = css`
  animation: arrow 1.5s ease-in-out infinite;
`;

const pageRootStyles = css`
  .swiper-button-prev,
  .swiper-button-next {
    transform: translateY(0);
    top: auto;
    right: auto;
    position: relative;
    margin-top: 0;
  }

  .vertical_mobile_progress {
    text-align: right;
    padding-right: 20px;
    z-index: 12;
    color: #fff;
  }
  .mobile_director {
    position: absolute;
    bottom: 0;
    left: 0;
    width: 100%;
    height: 45px;
    background: linear-gradient(
      180deg,
      rgba(51, 51, 51, 0) 0%,
      rgba(51, 51, 51, 0.6) 100%
    );
    z-index: 11;
  }
`;

const pageEditStyle = css`
  box-shadow: 0 0 0 3000px rgba(0, 0, 0, 0.4);
`;
