import { Dict_Imgs } from "@/dict";
import { createAttrsForm } from "../../defines/createAttrsForm";
import { InputNumber, Switch } from "ant-design-vue";
import { createColorOpts } from "../../defines/formOpts/createColorOpts";
import toolbarUI from "./toolbarUI";
import toolbarRight from "./toolbarRight";
import { DesignCompObj } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { RxValue } from "@/modules/editor/controllers/ReactCtrl/rxValue";

export { Component } from "./component";

export const options = {
  name: "多边形",
  thumbnail: require("@/modules/editor/assets/icons/polygon2.svg"),
};

const PolygonValue = {
  lineColor: "#000000",
  lineWidth: 0,
  isFill: true,
  fillColor: "#A4A4A4",
  points: [] as any,
  isClose: true,
  reverseFill: false,
}
export class CompPolygonObj extends DesignCompObj<typeof PolygonValue> {
  value = RxValue.create({...PolygonValue})
  onCreated() {
    this.compKey = "Polygon";
    this.layout.size = [750, 750]
  }
}

export function createComp() {
  return new CompPolygonObj();
}

export const Form = createAttrsForm([
  {
    label: "线宽",
    dataIndex: "value.lineWidth",
    component: InputNumber,
  },
  {
    label:"颜色",
    dataIndex: "value.lineColor",
    ...createColorOpts(),
  },
  {
    label: "是否封闭",
    dataIndex: "value.isClose",
    component: "Switch",
  },
  {
    label: "是否填充",
    dataIndex: "value.isFill",
    component: "Switch",
  },
  {
    label:"填充颜色",
    dataIndex: "value.fillColor",
    ...createColorOpts(),
    isVisible: (value, data) => data?.value?.isFill == true,
  },
  {
    label: "反向填充",
    dataIndex: "value.reverseFill",
    component: "Switch",
    isVisible: (value, data) => data?.value?.isFill == true,
  },
  {
    dataIndex: "value.points",
    component: toolbarRight,
  },
]);
