import { useEditor } from "@/modules/editor";
import { css } from "@linaria/core";
import { nextTick } from "process";
import { defineComponent, onMounted, onUnmounted, reactive, ref } from "vue";
import { string } from "vue-types";
import { View } from "../View";
import HeadlessEditor from "./EditorCustom";
import { useCompData } from "../../defines/hook";
import { CompTextObj } from ".";
import { Matrix } from "@/modules/editor/controllers/SelectCtrl/matrix";
import { Transform } from "@/modules/editor/controllers/SelectCtrl/objects/transform";
import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { Design_Page_Size } from "@/modules/editor/dicts/CompOptions";

export const Component = defineComponent({
  props: {
    compId: string().def(""),
  },
  setup(props) {
    const comp = useCompData<CompTextObj>(props.compId);
    const { store, actions, controls, helper } = useEditor();

    const state = reactive({
      editableId: "",
    });

    if (store.isEditMode) {
      actions.on("textFocus", function (compId, focus) {
        if (compId != props.compId) return;
        if (focus) {
          state.editableId = "" + Date.now();
          return;
        }
        state.editableId = "";
      });
    }

    return () => {
      //样式的scale转 width
      const m = Matrix.createFromMatrixStr(comp.layout.transformMatrix);
      const t = new Transform();
      t.setFromMatrix(m);
      m.ty = 0;
      m.tx = 0;
      m.rotate(-m.getRotate());

      const [w, h] = comp.layout.size || [Design_Page_Size[0], 0];

      const currWidth = w * t.scale.x;
      const textWidth = currWidth / t.scale.y;

      const width = helper.designToNaturalSize(currWidth);
      const textWidthSize = helper.designToNaturalSize(textWidth);
      //  const height = helper.designToNaturalSize(h * t.scale.y);
      m.invert();
      //m x m1 = m

      return (
        <View
          class={[textStyle]}
          compId={props.compId}
          onDblclick={() => {
            if (store.isEditMode) {
              state.editableId = "" + Date.now();
            }
          }}
        >
          <div
            style={{
              width: width,
              transform: `${m.getMatrixStr()}`,
              transformOrigin: "0 0",
            }}
          >
            <div
              style={{
                width: textWidthSize,
                transform: `scale(${t.scale.y})`,
                transformOrigin: "0 0",
              }}
            >
              {state.editableId ? (
                <EditorComp
                  compId={props.compId}
                  key={state.editableId}
                  onLost={() => {
                    state.editableId = "";
                    controls.textEditorCtrl.setCurrEditor(null);
                  }}
                />
              ) : (
                <div
                  innerHTML={comp.value.text}
                  id={`editor_${props.compId}`}
                  class={[textStyle, store.isEditMode && `pointer-events-none`]}
                />
              )}
            </div>
          </div>
        </View>
      );
    };
  },
});

const EditorComp = defineComponent({
  props: {
    compId: string().isRequired,
  },
  emits: ["lost"],

  setup(props, { emit }) {
    let editorRefVal = ref<HeadlessEditor>().value;
    let editorDomRef = ref<HTMLElement>();
    const comp = useCompData<CompTextObj>(props.compId);
    const { store, actions, helper, controls } = useEditor();
    const page = controls.pageCtrl;
    const gizmo = controls.selectCtrl.gizmo;

    let blurCanceler: any = null;

    onUnmounted(() => {
      blurCanceler?.();
    });
    const preHeight = ref<number>(0);
    const preTag = ref();
    const initHeight = () => {
      const dom: HTMLElement | null = document.querySelector(
        `#editor_${props.compId}`
      );
      if (!dom) {
        return false;
      }
      const h = helper.pxToDesignSize(dom.clientHeight);
      const isChange = Math.abs(preHeight.value - h) > 1;
      preHeight.value = h;
      if (isChange) {
        const size: any = comp.layout.size.slice(0);
        size[1] = h;
        comp.layout.size = size;
        helper.extendStreamCard(page.currStreamCardId);
      }
    };
    function isInCkBodyWrapper(dom: HTMLElement) {
      const out = { in: false, stop: true };

      if (editorRefVal) {
        const in1 =
          editorRefVal.ui.view.toolbar.element?.contains(dom) ||
          editorRefVal.ui.view.editable.element?.contains(dom);
        if (in1) {
          out.in = true;
          return out;
        }
        const ckBodyWrapper = document.querySelector(".ck-body-wrapper");
        if (ckBodyWrapper === dom || ckBodyWrapper?.contains(dom)) {
          out.in = true;
          return out;
        }
      }

      let n = 0;
      let curr: any = dom;
      do {
        if (
          curr.id == "text_toolform" ||
          curr.id == "toolbar" ||
          curr.classList.contains("editor_text_color") ||
          curr.classList.contains("ant-modal-input") ||
          curr.classList.contains("ant-select-dropdown")
        ) {
          out.in = true;
          out.stop = false;
          return out;
        }
        curr = curr.parentElement;
        n += 1;
        if (n > 10) break;
      } while (curr);
      return out;
    }

    function blurHandle() {
      function blur(e: MouseEvent) {
        const target = e.target as HTMLElement;

        if (!editorRefVal) return;
        const test = isInCkBodyWrapper(target);
        if (test.in) {
          if (test.stop) {
            e.stopPropagation();
          }
          return;
        }
        actions.submitUpdate();
        emit("lost");
      }

      document.addEventListener("mousedown", blur, {
        capture: true,
      });
      return () => {
        document.removeEventListener("mousedown", blur, { capture: true });
      };
    }

    const initEditor = async () => {
      // const dom: HTMLElement | null = document.querySelector(
      //   `#editor_${props.compId}`
      // );
      if (!editorDomRef.value) {
        return;
      }
      editorRefVal = await HeadlessEditor.create(editorDomRef.value);
      editorRefVal.setData(comp.value.text);
      editorRefVal.focus();
      const range = document.createRange();
      range.selectNodeContents(editorDomRef.value);
      const selection = window.getSelection();
      selection?.removeAllRanges();
      selection?.addRange(range);

      editorRefVal.model.document.on("change:data", () => {
        let value = editorRefVal?.getData();
        console.log(value);
        if (comp.value.text !== value) {
          if (!value) {
            preTag.value = comp.value.text;
          }
          if (!comp.value.text && preTag.value) {
            const text = value?.replace(/<[^>]+>/g, "");
            const spanTagReg =
              /([\s\S]*?<span[\s\S]*?>)([\s\S]*?)(<\/span>[\s\S]*?)/gi;

            value = preTag.value?.replace(spanTagReg, `$1${text}$3`);
            editorRefVal?.setData(value || "");
            editorRefVal?.focus();
            const selection = window.getSelection();
            nextTick(() => {
              if (selection?.focusNode) {
                selection?.setPosition(selection?.focusNode, text?.length);
              }
            });
          }
          actions.updateCompData(comp, "value.text", value);
          nextTick(() => {
            const element = editorRefVal?.ui.view.editable.element || null;
            if (!element) {
              return;
            }
            const h = helper.pxToDesignSize(element.clientHeight);
            const isChange = Math.abs(preHeight.value - h) > 1;
            preHeight.value = h;

            if (isChange) {
              const size: any = comp.layout.size.slice(0);
              if (
                gizmo.selected.length == 1 &&
                gizmo.selected[0].comp.compKey == "Group"
              ) {
                // 组内文本组件，更新组高度
                const currentComp = helper.findComp(props.compId);
                if (!currentComp) return;
                currentComp.setSize(size[0], h);
                helper.updateGroups(props.compId);
              } else {
                gizmo.selected[0].setSize(
                  helper.designSizeToPx(size[0]),
                  helper.designSizeToPx(h)
                );
              }
              helper.extendStreamCard(page.currStreamCardId);
            }
          });
        }
      });
      controls.textEditorCtrl.setCurrEditor(editorRefVal);
    };

    onMounted(() => {
      initEditor();
      blurCanceler = blurHandle();
      nextTick(() => {
        initHeight();
      });
    });
    return () => {
      return <div class={textStyle} ref={editorDomRef}></div>;
    };
  },
});

const textStyle = css`
  font-size: 14px;
  width: 100%;
  color: #666;
  word-break: break-word;
  h2 {
    color: #111111;
    font-size: 24px;
    margin-bottom: 0;
  }
  h3 {
    color: #333333;
    font-size: 18px;
    margin-bottom: 0;
  }
  p {
    margin: 0;
  }

  .ck.ck-editor__editable_inline {
    cursor: text;
    overflow: hidden;
    border: none !important;

    > :last-child,
    > :first-child {
      margin-top: 0;
      margin-bottom: 0;
    }
    padding: 0 !important;
  }
`;
