import { useCtx } from "@/comm/ctx";
import { useEditor } from "@/modules/editor";
import { css } from "@linaria/core";
import { IconPicture } from "@queenjs/icons";
import { defineComponent, onMounted, ref, watch } from "vue";
import { string } from "vue-types";
import { CompVideoObj } from ".";
import { useCompData } from "../../defines/hook";
import { View } from "../View";

export const Component = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props) {
    const { store, controls, actions, helper } = useEditor();
    const comp = useCompData<CompVideoObj>(props.compId);
    const { value } = comp;
    const videoRef = ref<HTMLVideoElement>();
    const ctx = useCtx();
    async function changeVal() {
      try {
        const url = await controls.pickCtrl.pickOneImage();
        if (!url) return;
        value.url = url;
        controls.pageCtrl.saveAssetsLocal(comp);
      } catch (error) {
        console.log(error);
      }
    }
    controls.pageCtrl.saveAssetsLocal(comp);
    function updateSize() {
      const { videoWidth, videoHeight } = videoRef.value as HTMLVideoElement;
      if (videoHeight == 0 || videoWidth == 0) return;
      const t = value.ratio || (videoWidth * 1.0) / videoHeight;
      comp.setH(comp.getW() / t);
      actions.onCompLayoutUpdated(comp);
      helper.extendStreamCard(controls.pageCtrl.currStreamCardId);
    }

    watch(
      () => [value.ratio],
      () => {
        updateSize();
      }
    );
    onMounted(() => {
      const el = videoRef.value as HTMLVideoElement;

      if (store.isEditMode) {
        el.addEventListener("loadedmetadata", () => {
          const { videoWidth, videoHeight } =
            videoRef.value as HTMLVideoElement;
          if (videoHeight != 0 && videoHeight != 0) {
            const r = (videoWidth * 1.0) / videoHeight;
            if (
              comp.value.ratio == 0 &&
              Math.abs(comp.layout.size[0] / comp.layout.size[1] - r) > 0.01
            ) {
              updateSize();
              return;
            }
          } else {
            setTimeout(() => {
              const { videoWidth, videoHeight } =
                videoRef.value as HTMLVideoElement;
              const r = (videoWidth * 1.0) / videoHeight;

              console.log(
                "loadedmetadata==>",
                videoWidth,
                videoHeight,
                comp.value.ratio,
                r,
                comp.layout.size
              );

              if (
                comp.value.ratio == 0 &&
                Math.abs(comp.layout.size[0] / comp.layout.size[1] - r) > 0.01
              ) {
                updateSize();
                console.log("updated sizeed");
              }
            }, 1000);
          }
        });
      }

      el.addEventListener("play", () => {
        controls.mediaCtrl.pauseOtherMedia(props.compId);
      });
      controls.mediaCtrl.setMediasInstance(props.compId, videoRef.value);
    });
    return () => {
      const options: any = {};
      if (value.autoplay) options.autoplay = false;
      if (value.loop) options.loop = true;
      if (value.controls) options.controls = true;
      if (value.poster) options.poster = value.poster;
      if (ctx.prjCtrl.isLocalUrl(value.url)) {
        value.url = ctx.prjCtrl.getHttpAbsoluteUri(value.url);
      }
      return (
        <View class={rootCls} compId={props.compId}>
          <video
            ref={videoRef}
            key={value.url}
            class="w-full object-cover"
            style={{ aspectRatio: value.ratio }}
            {...options}
          >
            <source src={value.url}></source>
          </video>
          {store.isEditMode && (
            <IconPicture class={btnCls} onClick={changeVal} />
          )}
        </View>
      );
    };
  },
});

const btnCls = css`
  display: none;
  position: absolute;
  top: 50%;
  left: 50%;
  padding: 20px;
  font-size: 37.5px;
  color: #666;
  border-radius: 50%;
  background-color: rgba(255, 255, 255, 0.7);
  transform: translate(-50%, -50%);
  @apply shadow;
`;

const rootCls = css`
  &:hover {
    .${btnCls} {
      display: block;
    }
  }
`;
