import { cx } from "@linaria/core";
import { Divider } from "ant-design-vue";
import { defineComponent, onMounted, reactive } from "vue";
import { any, bool } from "vue-types";
import ColorList, { gradientColorType } from "./ColorList";

const pureColors = [
  "#FFFFFF",
  "#D5D5D5",
  "#929292",
  "#5E5E5E",
  "#333333",
  "#222222",
  "#000000",
  "#47A2F8",
  "#6BE3CF",
  "#81D453",
  "#F9D958",
  "#ED6D57",
  "#EB559F",
  "#A055EB",
];

const gradientColors: gradientColorType[] = [
  {
    deg: 90,
    type: "linear",
    colors: ["#47A2F8", "#73C0F9"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#6BE3CF", "#98F9EA"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#81D453", "#A4F56A"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#F9D958", "#FDE582"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#ED6D57", "#F19B91"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#EB559F", "#F19BC8"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#A055EB", "#C99BF1"],
  },
];

const tabs = ["近期使用", "色彩库"];

export default defineComponent({
  props: {
    value: any<string | gradientColorType>().isRequired,
    showGradient: bool().def(true),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const state = reactive({
      latestColors: [],
      activeKey: 0,
    }) as any;

    const selectColor = (data: any) => {
      emit("change", data);
    };

    onMounted(() => {
      const list = localStorage.getItem("latestColors");
      let colors = list ? JSON.parse(list) : ["#fff"];
      if (!props.showGradient) {
        colors = colors.filter((d: any) => typeof d == "string");
      }
      colors.splice(14);
      state.latestColors = colors;
    });

    return () => {
      const { showGradient } = props;
      return (
        <div class="w-280px px-5px py-10px rounded">
          <Divider />
          <div class="mb-15px space-x-15px">
            {tabs.map((d, i) => (
              <span
                key={i}
                onClick={() => (state.activeKey = i)}
                class={cx(
                  "mt-15px mb-10px hover:opacity-80 transition cursor-pointer",
                  state.activeKey == i ? "text-light-50" : "text-gray"
                )}
              >
                {d}
              </span>
            ))}
          </div>
          {state.activeKey == 0 ? (
            <ColorList
              columns={7}
              value={props.value}
              data={state.latestColors}
              onChange={(data) => emit("change", data)}
            />
          ) : (
            <>
              <ColorList
                columns={7}
                value={props.value}
                data={pureColors}
                onChange={selectColor}
              />
              {showGradient && (
                <ColorList
                  columns={7}
                  class="mt-16px"
                  value={props.value}
                  data={gradientColors}
                  onChange={selectColor}
                />
              )}
            </>
          )}
        </div>
      );
    };
  },
});
