import { cx } from "@linaria/core";
import { Tag } from "ant-design-vue";
import { upperFirst } from "lodash";
import { Container, Draggable } from "vue-dndrop";
import { number, string } from "vue-types";

import Empty from "@/components/Empty";
import { useEditor } from "@/modules/editor";
import { useResource } from "@/modules/resource";
import { useAuth } from "@queenjs-modules/auth";
import { Image, Loadmore } from "@queenjs/ui";
import { defineUI, queenApi } from "queenjs";
import { onMounted, watch } from "vue";
import ListFilter from "./ListFilter";
import Menu from "./Menu";

export default defineUI({
  props: {
    sourceType: string<"comp" | "text" | "shape">().def("comp"),
    currentIndex: number(),
    contentIndex: number(),
  },
  setup(props) {
    const editor = useEditor();
    const auth = useAuth();
    const resource = useResource();

    function getData() {
      const ctrl = getCtrl();
      ctrl.hasLimit = true;
      ctrl.loadPage(1);
    }
    //@ts-ignore
    let isSys = (auth.store.userInfo.roles || []).indexOf("system") > -1;
    onMounted(() => getData());
    function getCtrl() {
      const key = `cust${upperFirst(props.sourceType)}ListCtrl`;
      //@ts-ignore
      return resource.controls[key];
    }
    watch(
      () => props.currentIndex,
      () => {
        if (props.currentIndex == props.contentIndex) {
          const ctrl = getCtrl();
          if (ctrl.state.list.length == 0) getData();
        }
      }
    );
    const filterChange = (v: any) => {
      const ctrl = getCtrl();
      const query = ctrl.state.query || {};
      ctrl.state.query = {
        ...query,
        categories: v,
      };
      ctrl.loadPage(1);
    };
    return () => {
      const { sourceType } = props;
      const ctrl = getCtrl();
      const dataSource = ctrl.state.list;

      return (
        <div>
          <ListFilter
            class={"my-16px"}
            sourceType={sourceType}
            isSys={false}
            onChange={filterChange}
          />
          <Container
            class={cx(
              "grid gap-10px",
              sourceType == "comp" ? "grid-cols-1" : "grid-cols-2"
            )}
            behaviour="copy"
            group-name="canvas"
            animation-duration={0}
            get-child-payload={(index: number) => {
              return {
                type: "CompCard",
                data: { id: dataSource[index]._id, isSys: isSys },
              };
            }}
          >
            {dataSource.map((item: compType) => {
              const items = ["编辑", "删除"];

              if (isSys) {
                item.published
                  ? items.push("取消发布")
                  : items.push("发布平台");
                // items.push("替换封面");
              }
              return (
                <Draggable>
                  <div
                    class="text-center leading-50px bg-light-50 rounded draggable-item relative cursor-pointer"
                    key={item._id}
                    title={item.title}
                  >
                    <Image
                      class="w-1/1 h-1/1 !object-contain"
                      src={item.thumbnail}
                      style={{ aspectRatio: 1 }}
                      onClick={() => {
                        editor.actions.clickCompUserToDesign(item._id, isSys);
                      }}
                      size={240}
                    />
                    {isSys && (
                      <Tag
                        color={item.published ? "green" : "#E88B00"}
                        // color="rgba(0, 0, 0, 0.4)"
                        class="absolute top-0 left-0 z-1 !rounded-none"
                      >
                        {item.published ? "已发布" : "未发布"}
                      </Tag>
                    )}
                    <div class="item_footer rounded-b-4px flex items-center justify-between p-4px bg-dark-50">
                      <div class="flex-1 w-0">
                        {/* <div class="text-white text-bold truncate">{item.title}</div> */}
                        <div class="flex items-center text-opacity-60 text-white text-12px">
                          {item.title}
                        </div>
                      </div>
                      <Menu
                        items={items}
                        onMenu={async (name) => {
                          // console.log("name", name);
                          if (name == "删除") {
                            const title = {
                              comp: "组合",
                              text: "文字",
                              shape: "形状",
                            };
                            await resource.actions.deleteUserComp(
                              item,
                              title[sourceType]
                            );
                            ctrl.fresh();
                            queenApi.messageSuccess("删除成功!");
                            return;
                          }
                          if (name == "替换封面") {
                            const url =
                              await editor.controls.pickCtrl.pickOneImage();
                            if (!url) return;
                            await resource.https.updateComp({
                              _id: item._id,
                              thumbnail: url,
                            });
                            item.thumbnail = url;
                            queenApi.messageSuccess("替换成功");

                            return;
                          }
                          if (name == "编辑") {
                            await resource.actions.editSource(item, sourceType);
                            ctrl.fresh();
                            return;
                          }

                          const pub = name == "发布平台";
                          await resource.actions.publishFrame(item, pub);
                          item.published = pub;
                        }}
                      />
                    </div>
                  </div>
                </Draggable>
              );
            })}
          </Container>
          {dataSource.length == 0 ? (
            <Empty class="pt-150px" />
          ) : (
            <Loadmore
              class="mt-20px"
              loading={ctrl.state.loading}
              canLoad={ctrl.state.canLoadNext}
              onChange={ctrl.loadNextPage}
            />
          )}
        </div>
      );
    };
  },
});
