import { css } from "@linaria/core";
import { Cascader } from "ant-design-vue";
import { computed, defineComponent } from "vue";
import { any, string } from "vue-types";

export default defineComponent({
  props: {
    label: string().isRequired,
    data: any().isRequired,
    value: any().isRequired,
  },
  emits: ["change"],
  setup(props, { emit }) {
    const selected = computed(() => {
      return getCheckedArr(props.data, props.value, []) || [];
    });

    function getCheckedArr(list: any, value: any, pre: any): any {
      if (!list || !list.length) return;
      list = [...list];
      const item = list.pop();
      if (!item) return;

      if (item.id == value) {
        return [...pre, item];
      } else {
        return (
          getCheckedArr(item.children, value, [...pre, item]) ||
          getCheckedArr(list, value, pre)
        );
      }
    }
    return () => (
      <Cascader
        class={rootStyle}
        options={props.data}
        value={selected.value.map((d: any) => d.id)}
        placeholder={props.label}
        expand-trigger="hover"
        changeOnSelect={true}
        fieldNames={{ label: "name", value: "id", children: "children" }}
        onChange={(values) => {
          emit("change", values ? values[values.length - 1] : undefined);
        }}
      >
        {{
          displayRender: () => {
            const value = selected.value[selected.value.length - 1]?.name;
            return value ? value : props.label;
          },
        }}
      </Cascader>
    );
  },
});

const rootStyle = css`
  &.ant-cascader {
    .ant-select-selector {
      background-color: #303030;
      border-color: transparent;
    }
    .ant-select-selection-placeholder {
      color: rgba(255, 255, 255, 0.5);
    }
    .ant-select-clear {
      color: rgba(255, 255, 255, 1);
    }
  }
`;
