import { useResource } from "@/modules/resource";
import { css } from "@linaria/core";
import { cloneDeep } from "lodash";
import { defineComponent, reactive } from "vue";
import { bool, string } from "vue-types";
import CateFilter from "./CateFilter";
import CateSelect from "./CateSelect";

export default defineComponent({
  props: {
    sourceType: string(),
    isSys: bool().def(true),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const { controls } = useResource();
    const state = reactive({
      limitLen: 2,
      values: {} as any,
    });
    const getCate = () => {
      const categories = props.isSys
        ? controls.categoryCtrl.state.syscategories
        : controls.categoryCtrl.state.categories;
      const currCate: any = categories.find((e: any) => {
        return e.value == props.sourceType?.toLowerCase();
      });

      return currCate?.children || [];
    };
    const cateChange = (id: string, v: string) => {
      const values = cloneDeep(state.values);
      if (!v && values[id]) {
        delete values[id];
      } else {
        values[id] = v;
      }
      state.values = values;
      const categories = Object.values(values);
      emit(
        "change",
        categories && categories.length > 0 ? categories : undefined
      );
    };
    const filterChange = (v: any) => {
      let values = cloneDeep(state.values);
      if (!v) {
        emit("change", undefined);
        state.values = {};
        return;
      }
      values = { ...values, ...v };
      Object.keys(values).map((key: any) => {
        if (values[key] == undefined) {
          delete values[key];
        }
      });
      state.values = values;
      const categories = Object.values(values);
      emit(
        "change",
        categories && categories.length > 0 ? categories : undefined
      );
    };
    return () => {
      const options = getCate();
      if (options.length == 0) return null;
      return (
        <div class={rootStyle}>
          {/* {options.slice(0, state.limitLen).map((d: any, i: number) => (
            <div class={"cate_item flex-1"}>
              <CateSelect
                key={i}
                label={d.name}
                data={d.children}
                value={state.values[d.id]}
                onChange={(v) => cateChange(d.id, v)}
              />
            </div>
          ))} */}

          <div class={"flex-1"}>
            <CateFilter
              options={options}
              value={state.values}
              onChange={filterChange}
            />
          </div>
        </div>
      );
    };
  },
});

const rootStyle = css`
  display: flex;
  align-items: center;
  .ant-cascader {
    width: 100%;
  }
  .cate_item {
    margin-left: 10px;
    width: 90px;
    &:first-child {
      margin-left: 0;
    }
  }
`;
