import { defineComponent, onMounted, reactive } from "vue";
import { Container, Draggable } from "vue-dndrop";

import { useCtx } from "@/comm/ctx";
import Empty from "@/components/Empty";
import { useEditor } from "@/modules/editor";
import { nanoid } from "nanoid";
import SourceItem from "./Sources/SourceItem";
import { queenApi } from "queenjs";
export default defineComponent({
  setup(props) {
    const editor = useEditor();
    // const resource = useResource();
    // provide("sourceFrom", props.sourceFrom);
    const state = reactive({
      renderImages: [] as any,
    });
    const ctx = useCtx();
    function clickToDesign(url: string) {
      editor.actions.addImageToDesign(url);
    }

    function getData() {
      console.log(ctx.prjCtrl.getOutputDir());
      ctx.deviceCtrl
        .GetFilesInDir(ctx.prjCtrl.getOutputDir(), "\\.png$")
        .then((ret: any) => {
          ret &&
            ret.map((e: any) => {
              const relaUrl = ctx.prjCtrl.getRelativeUri(e.Fpath);
              const absoUrl = ctx.prjCtrl.getHttpAbsoluteUri(relaUrl);
              state.renderImages.push({
                _id: nanoid(),
                file: {
                  url: absoUrl,
                  size: e.Size,
                },
                fileType: "image",
                createTime: e.CreateAt * 1000,
              });
            });
        });
    }
    const deleteImage = async (index: number) => {
      await queenApi.showConfirm({
        title: "删除提示",
        content: "确认删除当前渲染图？",
        type: "danger",
      });
      const ctx = useCtx();
      const relaUrl = ctx.prjCtrl.getRelativeUri(
        state.renderImages[index].file.url
      );
      const fpath = ctx.prjCtrl.getLocalAbsoluteUri(relaUrl);
      const ok = await ctx.deviceCtrl.RemoveFile(fpath);
      if (!ok) {
        queenApi.messageError("删除失败");
        return;
      }
      let s = state.renderImages;
      s.splice(index, 1);
      state.renderImages = s;

      queenApi.messageSuccess("删除成功");
    };
    onMounted(() => getData());

    return () => {
      //   const control = getCurrCtrl();
      const dataSource = state.renderImages;

      return (
        <div>
          <Container
            class="grid grid-cols-2 gap-10px"
            behaviour="copy"
            group-name="canvas"
            animation-duration={0}
            get-child-payload={(index: number) => {
              return {
                type: "Image",
                data: dataSource[index],
              };
            }}
          >
            {dataSource.map((item: any, index: number) => (
              <Draggable key={item._id}>
                <SourceItem
                  class="draggable-item cursor-pointer"
                  record={item}
                  isVideo={false}
                  tagable={true}
                  onClick={() => clickToDesign(item.file.url)}
                  hasEdit={false}
                  onMenu={async (name) => {
                    console.log(name);
                    if (name == "delete") {
                      deleteImage(index);
                    }
                  }}
                />
              </Draggable>
            ))}
          </Container>
          {dataSource.length == 0 && <Empty class="pt-150px" />}
        </div>
      );
    };
  },
});
