import { defineComponent, onMounted, provide } from "vue";
import { Container, Draggable } from "vue-dndrop";
import { string } from "vue-types";

import Empty from "@/components/Empty";
import { useEditor } from "@/modules/editor";
import { useResource } from "@/modules/resource";
import { Loadmore } from "@queenjs/ui";
import { Button } from "ant-design-vue";
import { queenApi } from "queenjs";
import SourceItem from "./SourceItem";

import ListFilter from "../ListFilter";

export const Sources = defineComponent({
  props: {
    sourceType: string<"Image" | "Video">().def("Image"),
    sourceFrom: string<"system" | "user">().def("user"),
  },
  setup(props) {
    const editor = useEditor();
    const resource = useResource();
    provide("sourceFrom", props.sourceFrom);

    function getCurrCtrl() {
      // sysImageListCtrl sysVideoListCtrl custImageListCtrl custVideoListCtrl
      if (props.sourceFrom == "user")
        return resource.controls[
          props.sourceType === "Image"
            ? "custImageListCtrl"
            : "custVideoListCtrl"
        ];
      return resource.controls[
        props.sourceType === "Image" ? "sysImageListCtrl" : "sysVideoListCtrl"
      ];
    }

    function clickToDesign(url: string) {
      if (props.sourceType == "Image") {
        editor.actions.addImageToDesign(url);
        return;
      }

      editor.actions.clickCompToDesign(props.sourceType, (comp) => {
        comp.value.url = url;
      });
    }

    function getData() {
      const ctrl = getCurrCtrl();
      ctrl.hasLimit = true;
      ctrl.loadPage(1);
    }

    onMounted(() => getData());

    const filterChange = (v: any) => {
      const ctrl = getCurrCtrl();
      const query = ctrl.state.query || {};
      ctrl.state.query = {
        ...query,
        categories: v,
      };
      ctrl.loadPage(1);
    };
    return () => {
      const control = getCurrCtrl();
      const dataSource = control.state.list;
      const { sourceType } = props;

      return (
        <div>
          {props.sourceFrom == "user" && (
            <div class="sticky top-0 mt-6px mb-16px z-11 bg-[#262626]">
              <Button
                block
                type="primary"
                onClick={async () => {
                  await resource.actions.createMaterial();
                  control.loadPage(1);
                }}
              >
                上传素材
              </Button>
            </div>
          )}
          <div>
            <ListFilter
              class={"mb-16px"}
              sourceType={props.sourceType}
              isSys={props.sourceFrom == "user" ? false : true}
              onChange={filterChange}
            />
            <Container
              class="grid grid-cols-2 gap-10px"
              behaviour="copy"
              group-name="canvas"
              animation-duration={0}
              get-child-payload={(index: number) => {
                return {
                  type: props.sourceType,
                  data: dataSource[index],
                };
              }}
            >
              {dataSource.map((item: any) => (
                <Draggable key={item._id}>
                  <SourceItem
                    class="draggable-item cursor-pointer"
                    record={item}
                    isVideo={sourceType == "Video"}
                    tagable={props.sourceFrom == "user"}
                    onClick={() => clickToDesign(item.file.url)}
                    onMenu={async (name) => {
                      console.log(name);
                      if (name == "delete") {
                        await resource.actions.deleteMaterial(item);
                        control.loadPage(1);
                        queenApi.messageSuccess("删除成功!");
                        return;
                      }
                      if (name == "edit") {
                        await resource.actions.editSource(item, sourceType);
                        control.loadPage(1);
                        return;
                      }
                      if (name == "publish" || name == "unpublish") {
                        const pub = name == "publish";
                        await resource.actions.publishMaterial(item, pub);
                        item.published = pub;
                        queenApi.messageSuccess("操作成功!");
                      }
                    }}
                  />
                </Draggable>
              ))}
            </Container>
            {dataSource.length == 0 ? (
              <Empty class="pt-150px" />
            ) : (
              <Loadmore
                class="mt-20px"
                loading={control.state.loading}
                canLoad={control.state.canLoadNext}
                onChange={control.loadNextPage}
              />
            )}
          </div>
        </div>
      );
    };
  },
});
