import List from "@/components/AssetsList";
import { useResource } from "@/modules/resource";
import { PromotionController } from "@/modules/resource/controllers/PromotionController";
import { useAuth } from "@queenjs-modules/auth";
import { defineUI } from "queenjs";
import { computed, onMounted, reactive } from "vue";
import { any } from "vue-types";
import Filter from "./CategoryFilter";
import Header from "./Header";
import PromotionItem from "./PromotionItem";

export default defineUI({
  props: {
    Controller: any<PromotionController>().isRequired,
  },
  slots: {
    Header,
    Filter,
    List,
  },
  setup(props, { slots }) {
    const auth = useAuth();
    const resource = useResource();

    const state = reactive({
      categoryList: computed(() => {
        const categories = resource.controls.categoryCtrl.state.categories;
        const currCate: any = categories.find((e: any) => {
          return e.value == "template";
        });
        if (!currCate) return [];
        const r = currCate.children.find((e: any) => e.name == "平台");
        return r?.children || [];
      }),
      pcCategoryId: computed(() => {
        const item: any = state.categoryList.find((e: any) => e.value == "PC");
        return item.id;
      }),
    });

    const FilterChange = (query: any) => {
      const ctrlQuery = props.Controller.ListCtrl.state.query || {};
      props.Controller.ListCtrl.state.query = {
        ...ctrlQuery,
        categories: query,
      };
      props.Controller.ListCtrl.loadPage(1);
    };

    function isPcDesign(record: any) {
      let res = false;
      const categories: any[] = record.categories || [];
      if (categories.includes(state.pcCategoryId)) res = true;
      return res;
    }

    onMounted(() => {
      props.Controller.ListCtrl.state.query = {};
      props.Controller.ListCtrl.loadPage(1);
    });

    return () => {
      //@ts-ignore
      const isSys = (auth.store.userInfo.roles || []).indexOf("system") > -1;

      return (
        <div>
          <slots.Header onAdd={props.Controller.createPromotion} />
          <slots.Filter
            sourceType="template"
            onAdd={props.Controller.createPromotion}
            onChange={FilterChange}
          />
          <slots.List
            gap="25px"
            class="my-30px"
            columns={5}
            control={props.Controller.ListCtrl}
            item={(record: any) => {
              const isPc = isPcDesign(record);
              return (
                <PromotionItem
                  isPc={isPc}
                  record={record}
                  onMenu={(name) => {
                    props.Controller.onMenuClick(name, record, isSys);
                  }}
                  onEdit={(record) => props.Controller.onEdit(record)}
                />
              );
            }}
          />
        </div>
      );
    };
  },
});
